/* **************************************************************** */
/* barrier.c, sample test for csc 4/256 project 4/5                 */
/*                                                                  */
/* Copyright 2002, University of Rochester Department of Computer   */
/* Science; may be freely re-used provided that this header remains */
/* intact.                                                          */
/* **************************************************************** */

/* Compile with:
   gcc -c -g barrier.c
   gcc -o barrier -g -msupersparc -Wa,-xarch=v8plus -O3 -DGCC -lrt barrier.o dsm.o
*/

#include <stdio.h>
#include <stdlib.h>
#include "dsm.h"

/* prototypes */
void *worker(void *ID);

/* main */
int main(int argc, char** argv)
{
  int idx;
  int myID;

/*create memory block*/
    dsm_create(4, 512);

  if (1 == argc)
  {
      printf("Usage: barrier <thread ID 0..7>\n");
      exit(-1);
  }

  /* Retrieve ID number */
  myID = atoi(argv[1]);
  if (0 > myID || 7 < myID)
  {
      printf("Usage: barrier <thread ID 0..7>\n");
      exit(-1);
  }

  // Run as a worker ourselves
  (void)worker((void *)myID);

/*destroy shared memory reference*/
  dsm_destroy();

  return 0;
}

#define CRASHWRITE(i,v,s)    if (0 == dsm_write(i,v,s)) goto failed
#define CRASHREAD(i,v,s)    if (0 == dsm_read(i,v,s)) goto failed

/* A simple barrier that requires no RMW atomic primitives */
/* Note: this is very inefficient! */
void barrier(int myID, int *sense)
{
    int count, idx, cur;
    CRASHWRITE(myID, sense, 1);

    do
    {

	count = 0;
	for (idx = 0; idx < 8; idx++)
	{
	    CRASHREAD(idx, &cur, 1);
	    count += (cur == *sense) || (cur == ((*sense + 1) % 3));
	}	
    }
    while (count < 8);

    /* rotate sense for next time */
    /* Note that we can't just flip between two values because that
       leads to a race condition! */
    *sense = *sense + 1;
    if (*sense == 3)
	*sense = 0;
    return;

  failed:
    printf("DSM I/O failure for thread %d: bailing.\n", myID);
    pthread_exit(0);
}

void verify(void)
{
    int cur, idx;

    /* By the time we get here, all values from 17 to 24 */
    /* should contain the value 36 */
    for (idx = 0; idx < 8; idx++)
    {
	CRASHREAD(idx + 17, &cur, 1);
	printf("Address %d: sum %d\n", idx + 17, cur);
    }
    return;

  failed:
    printf("DSM I/O failure during verification.\n");
}

void *worker(void *ID)
{
    int myID = (int)ID;
    int count;
    int sense = 1;
    int cur;
    int offset;
    int  v[1];
    
    for (count = 0; count < 8; count++)
    {
	barrier(myID, &sense);

	offset = (count + myID) % 8;
	CRASHREAD(17 + offset, &cur, 1);
	v[0] = cur+count+1;
	CRASHWRITE(17 + offset, v, 1);

	if (count == myID)
	{
	    printf("Worker %d reporting completion of barrier %d.\n", 
		   myID, count);
	}
    }

    /* One last barrier to make sure everyone's done */
    barrier(myID, &sense);
    if (0 == myID)
	verify();

    return 0;

  failed:
    printf("DSM I/O failure for thread %d: bailing.\n", myID);
}

/* EOF */












