/* **************************************************************** */
/* iftest.c, interface test for csc 4/256 project 4/5               */
/*                                                                  */
/* Copyright 2002, University of Rochester Department of Computer   */
/* Science; may be freely re-used provided that this header remains */
/* intact.                                                          */
/* **************************************************************** */

/* Compile with:
   gcc -c -g iftest.c
   gcc -o iftest -lpthread -g -msupersparc -Wa,-xarch=v8plus -O3 -DGCC -lrt iftest.o dsm.o
*/

#include <stdio.h>
#include <stdlib.h>
#include "dsm.h"

#define TESTFAILED(msg)    { printf(msg); failcount++; }
int failcount = 0;

/* main */
int main(int argc, char** argv)
{
    int old, new, t, i;

    /*create memory block*/
    dsm_create(4, 64);

    /* test handling of null pointer */
    if (0 != dsm_read(0, NULL, 1))
	TESTFAILED("Bad pointer not handled correctly.\n")

    /* test range handling */
    if (0 != dsm_read(-1, &t, 1) || (0 != dsm_read(1024*1024*16, &t, 1)))
	TESTFAILED("Bad address for dsm_read() not checked.\n")
    if (0 != dsm_write(-1, &t, 1) || (0 != dsm_write(1024*1024*16, &t, 1)))
	TESTFAILED("Bad address for dsm_write() not checked.\n")

    /* test memory prefilling: shared memory should be zero initially */
    for (i = 0; i < 128; i++)
    {
	if (1 != dsm_read(i, &t, 1))
	    TESTFAILED("Valid read attempt failed.\n")
	else if (0 != t)
	{
	    TESTFAILED("Memory not initialized to zero")
	    break;
	}
    }

    /* Validate that we can read back what we write to memory */
    old = (int)0XFACED;
    if (1 != dsm_write(17, &old, 1))
	TESTFAILED("Valid write attempt failed.\n")
    if (1 != dsm_read(17, &new, 1))
	TESTFAILED("Valid read attempt failed.\n")
	  printf("old = %d new=%d\n", old, new);
    if (old != new)
	TESTFAILED("Read/write function not working.\n")

    /* That's it for now! */
    if (0 == failcount)
	printf("Congratulations: all tests passed.\n");
    else
	printf("Bummer: %d test cases failed.\n");

/*destroy shared memory reference*/
  dsm_destroy();

    return 0;
}

/* EOF */
