/* **************************************************************** */
/* pingpong.c, sample test for csc 4/256 project 4/5                */
/*                                                                  */
/* Copyright 2002, University of Rochester Department of Computer   */
/* Science; may be freely re-used provided that this header remains */
/* intact.                                                          */
/* **************************************************************** */

/* Compile with:
   gcc -c -g pingpong.c
   gcc -o pingpong -g -msupersparc -Wa,-xarch=v8plus -O3 -DGCC -lrt pingpong.o dsm.o
*/

#include <stdio.h>
#include <stdlib.h>
#include "dsm.h"

/* prototypes */
void *tweedledum(void *dummy);
void *tweedledee(void *dummy);

#define TWEEDLEDUM 0

/* main */
int main(int argc, char** argv)
{
  int idx, err, who;

/*create memory block*/
    dsm_create(4, 512);

  if (1 == argc)
  {
      printf("Usage: pingpong <0 or 1>\n");
      printf("Tweedledum is 0, Tweedledee is 1.\n");
      exit(-1);
  }
  who = atoi(argv[1]);
  if (0 != who && 1 != who)
  {
      printf("Usage: pingpong <0 or 1>\n");
      printf("Tweedledum is 0, Tweedledee is 1.\n");
      exit(-1);
  }

  if (who == TWEEDLEDUM)
      (void)tweedledum(0);
  else
      (void)tweedledee(0);

/*destroy shared memory reference*/
  dsm_destroy();

  return 0;
}

void *tweedledum(void *dummy)
{
    int count = 10;
    int temp, arg, newarg;
    int v[1];

    while (count-- > 0)
    {
	/* Wait until tweedledee shuts up */
	do
	{
	    if (1 != dsm_read(0, &temp, 1))
	    {
		printf("tweedledum: read failure! exiting.\n");
		return 0;
	    }
	}
	while (0 != temp);

	/* Read his argument */
	if (1 != dsm_read(1, &arg, 1))
	{
	    printf("tweedledum: read failure! exiting.\n");
	    return 0;
	}

	/* Grumble */
	newarg = arg - 7;
	if (newarg < 0)
	    newarg = -1 * newarg;
	printf("tweedledum: tweedledee says %d? Hah, I say %d.\n", 
	       arg, newarg);

	/* Argue back */
	if (1 != dsm_write(1, &newarg, 1))
	{
	    printf("tweedledum: write failure! exiting.\n");
	    return 0;
	}

	/* Give tweedledee a chance to argue */
	v[0] = 1-temp;
	if (1 != dsm_write(0, v, 1))
	{
	    printf("tweedledum: write failure! exiting.\n");
	    return 0;
	}
    }
}

void *tweedledee(void *dummy)
{
    int count = 10;
    int temp, arg, newarg;
    int v[1];

    while (count-- > 0)
    {
	/* Wait until tweedledum shuts up */
	do
	{
	    if (1 != dsm_read(0, &temp, 1))
	    {
		printf("tweedledee: read failure! exiting.\n");
		return 0;
	    }
	}
	while (1 != temp);

	/* Read his argument */
	if (1 != dsm_read(1, &arg, 1))
	{
	    printf("tweedledee: read failure! exiting.\n");
	    return 0;
	}

	/* Grumble */
	newarg = arg - 5;
	if (newarg < 0)
	    newarg *= -1;
	printf("tweedledee: tweedledum says %d? Hah, I say %d.\n", 
	       arg, newarg);

	/* Argue back */
	if (1 != dsm_write(1, &newarg, 1))
	{
	    printf("tweedledee: write failure! exiting.\n");
	    return 0;
	}

	/* Give tweedledum a chance to argue */
	v[0] = 1-temp;
	if (1 != dsm_write(0, v, 1))
	{
	    printf("tweedledee: write failure! exiting.\n");
	    return 0;
	}
    }
}

/* EOF */













